#ifndef	V_POWER2_H
#define	V_POWER2_H

/*
 * Handy macros for making powers of 2.
 */

/*
 * Returns 1 if value is power of 2; otherwise returns 0
 */
#define IS_POWER_TWO(x)			(((x) & ((x)-1)) ? 0 : 1)

/*
 * Aligns u32_t 'src', so:
 *	a) dst >= src;
 *	b) least 'zero_bits' in 'dst' set to zero.
 * For expample, if 'zero_bits' is 2: 0->0; {1,2,3,4}->4; etc.
 * 'dst' and 'src' can be the same.
 */
#define	ALIGN_UP(dst,src,zero_bits) { \
		u32_t set_bits=(1<<(zero_bits))-1; \
		(dst)=((src)+set_bits)&(0xFFFFFFFF-set_bits); }

/*
 * Based on Quake's net_loop.c:IntAlign
 */
#define	INT_ALIGN(value) \
		(((value) + (sizeof(int) - 1)) & (~(sizeof(int) - 1)))

/*
 * Sets retval to power of 2 which is not less than 'value'
 * TODO: There are better ways to do it ...
 */
#define	NOT_LESS_POWER_TWO(retval,value,type)\
		{\
			type mask;\
			if((value)==(type)0) {\
				retval=0;\
			} else {\
				mask=(type)1 << ((sizeof(type)<<3)-1);\
				while(!(mask & (value)))\
					mask>>=1;\
				retval = (mask < (value)) ? mask << 1 : mask;\
			}\
		}

/*
 * Sets retval to power of 2 which is greater than 'value'.
 * TODO: There are better ways to do it ...
 */
#define	GREATER_POWER_TWO(retval,value,type)\
		{\
			type mask;\
			if((value)==(type)0) {\
				retval=1;\
			} else {\
				mask=(type)1 << ((sizeof(type)<<3)-1);\
				while(!(mask & (value)))\
					mask>>=1;\
				retval = (mask <= (value)) ? mask << 1 : mask;\
			}\
		}

#endif	/* V_POWER2_H */
