#ifndef V_TYPES_H
#define V_TYPES_H

/*
 *	Limits.
 */
#if     !defined(MAXCHAR)
#define	MAXCHAR							(0x7F)
#define	MINCHAR							(0x80)
#endif  /* !MAXCHAR */
#if     !defined(MAXSHORT)
#define	MAXSHORT						0x7FFF
#define	MINSHORT						0x8000
#endif  /* !MAXSHORT */
#if     !defined(MAXLONG)
#define MAXLONG							(0x7FFFFFFF)
#define MINLONG							(0x80000000)
#endif  /* !MAXLONG */
#if     !defined(MAXLONGLONG) && !defined(INT64BROKEN)
#define MAXLONGLONG						0x7FFFFFFFFFFFFFFF
#define MINLONGLONG						0x8000000000000000
#endif  /* MAXLONGLONG */
#if     !defined(MAXFLOAT)
#define MAXFLOAT						3.4e+38
#endif  /* MAXFLOAT */
#if     !defined(MAXDOUBLE)
#define MAXDOUBLE						1.7e+308
#endif  /* MAXDOUBLE */

#ifndef	DOUBLEVEC
#define	MAXVEC							((vec_t)MAXFLOAT)
#else
#define	MAXVEC							((vec_t)MAXDOUBLE)
#endif	/* DOUBLEVEC */

/*
 * Upper limit for estimated fp round-off error (and for square of error)
 */
#ifndef	FLOAT_ZERO_EPSILON
#define	FLOAT_ZERO_EPSILON				1e-6
#endif	/* FLOAT_ZERO_EPSILON */
#ifndef	FLOAT_ZERO_EPSILON_SQUARE
#define	FLOAT_ZERO_EPSILON_SQUARE		1e-12
#endif	/* FLOAT_ZERO_EPSILON_SQUARE */

/*
 * Base LibV types.
 */
typedef unsigned char			u8_t,	*u8_p;
typedef signed char				s8_t,	*s8_p;
typedef unsigned short			u16_t,	*u16_p;
typedef signed short			s16_t,	*s16_p;
typedef unsigned long			u32_t,	*u32_p;
typedef signed long				s32_t,	*s32_p;
#ifndef	INT64BROKEN
#if		!defined(__UNIX__) && !defined(__CYGWIN32__)
typedef unsigned __int64		u64_t,	*u64_p;
typedef signed __int64			s64_t,	*s64_p;
#else
typedef unsigned long long int	u64_t,	*u64_p;
typedef long long int			s64_t,	*s64_p;
#endif	/* __UNIX__ */
#endif	/* INT64BROKEN */

/*
 * Pointers: void and string.
 * Note: char_t always signed.
 */
typedef	void		void_t,	*void_p, **void_pp;
typedef signed char	char_t, *char_p, **char_pp;

/*
 * Replacement for size_t
 */
typedef	unsigned	vsize_t;

/*
 *	Handle of LibV object. Returned by numerous factory functions.
 */
typedef		void	*vhandle_t;

/*
 * Another one boolean type
 */
#if			!defined(__cplusplus) && defined(__WATCOMC__)
typedef		enum		{ false = 0, true = 1 }	vboolean;
#else		/* defined(__cplusplus) || !defined(__WATCOMC__) */
typedef 	int			vboolean;
#define 	false		0
#define 	true		1
#endif		/* defined(__cplusplus) || !defined(__WATCOMC__) */

/*
 * Base floating-point scalar type.
 * Names of type and macro stolen from Quake.
 */ 
#ifdef		DOUBLEVEC
typedef		double				vec_t, *vec_p;
#else		/* !DOUBLEVEC */
typedef		float				vec_t, *vec_p;
#endif		/* DOUBLEVEC  */

/*
 * Vector and matrix types (low order).
 * Values in matrices stored in row - major order (unlike OpenGL)
 */
typedef vec_t					vec2_t[2];
typedef vec_t					vec3_t[3];
typedef vec_t					vec4_t[4];
typedef vec_t					matrix2_t[2][2];
typedef vec_t					matrix3_t[3][3];
typedef vec_t					matrix4_t[4][4];
typedef vec_t					matrix3x4_t[3][4];

/*
 *	24- & 32-bit RGBA colors.
 */
typedef u8_t					color3b_t[3];
typedef u8_t					color4b_t[4];

/*
 * TODO: move these includes to libv.h
 */
#include	<misc/power2.h>
#include	<misc/bswap.h>

#endif	/* V_TYPES_H */
